"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Decorators_1 = require("../ioc/Decorators");
exports.HEALTH_CHECK_GROUP = 'inceptum:health';
function RegisterAsHealthCheck(target) {
    Decorators_1.RegisterInGroup(exports.HEALTH_CHECK_GROUP)(target);
    Decorators_1.Lazy(false)(target);
}
exports.RegisterAsHealthCheck = RegisterAsHealthCheck;
var HealthCheckStatus;
(function (HealthCheckStatus) {
    HealthCheckStatus["OK"] = "OK";
    HealthCheckStatus["NOT_READY"] = "NOT_READY";
    HealthCheckStatus["WARNING"] = "WARNING";
    HealthCheckStatus["CRITICAL"] = "CRITICAL";
})(HealthCheckStatus = exports.HealthCheckStatus || (exports.HealthCheckStatus = {}));
function getStatusIndex(status) {
    return [HealthCheckStatus.OK, HealthCheckStatus.NOT_READY, HealthCheckStatus.WARNING, HealthCheckStatus.CRITICAL].indexOf(status);
}
class HealthCheckResult {
    constructor(status, message, timestamp, data) {
        this.status = status;
        this.message = message;
        this.data = data;
        this.timestamp = timestamp || new Date().getTime();
    }
    static getInitialResult() {
        return new HealthCheckResult(HealthCheckStatus.NOT_READY, 'First check not run yet');
    }
    static staleResult(healthCheck, lastResult) {
        return new HealthCheckResult(HealthCheckStatus.CRITICAL, `Stale check result for check ${healthCheck.checkName}. Last result on ${lastResult.timestamp}`);
    }
}
exports.HealthCheckResult = HealthCheckResult;
class HealthCheck {
    constructor(checkName, sleepMillis = 60000, staleNumFails = 2) {
        this.checkName = checkName;
        this.sleepMillis = sleepMillis;
        this.staleNumFails = staleNumFails;
        this.started = false;
        this.lastResult = HealthCheckResult.getInitialResult();
    }
    getCheckName() {
        return this.checkName;
    }
    getSleepMillis() {
        return this.sleepMillis;
    }
    start() {
        if (this.started) {
            return;
        }
        this.started = true;
        this.doStart();
    }
    doStart() {
        this.timer = setInterval(() => { this.lastResult = this.doCheck(); }, this.getSleepMillis());
    }
    stop() {
        if (this.started && this.timer) {
            this.doStop();
        }
    }
    doStop() {
        clearInterval(this.timer);
    }
    isStarted() {
        return this.started;
    }
    getLastResult() {
        const threshold = (new Date().getTime()) - this.getSleepMillis() * this.staleNumFails;
        if (this.lastResult.timestamp < threshold) {
            return HealthCheckResult.staleResult(this, this.lastResult);
        }
        return this.lastResult;
    }
}
exports.HealthCheck = HealthCheck;
class HealthCheckGroup extends HealthCheck {
    constructor(groupName) {
        super(`Group: ${groupName}`, 0, 0);
        this.healthChecks = [];
        this.groupKey = groupName;
    }
    addCheck(healthCheck, asName) {
        this.addCheckAs(healthCheck, asName || healthCheck.getCheckName());
    }
    addCheckAs(healthCheck, asName) {
        if (asName.indexOf('.') >= 0) {
            const parts = asName.split('.', 2);
            const group = this.healthChecks.find((hc) => (hc instanceof HealthCheckGroup) && (hc.groupKey === parts[0]));
            if (group) {
                group.addCheckAs(healthCheck, parts[1]);
            }
            else {
                const newGroup = new HealthCheckGroup(parts[0]);
                newGroup.addCheckAs(healthCheck, parts[1]);
                this.addCheck(newGroup, parts[0]);
            }
        }
        else {
            this.healthChecks.push(healthCheck);
            if (this.isStarted()) {
                healthCheck.start();
            }
        }
    }
    doStart() {
        this.healthChecks.forEach((healthCheck) => {
            healthCheck.start();
        });
    }
    doStop() {
        this.healthChecks.forEach((healthCheck) => {
            healthCheck.stop();
        });
    }
    doCheck() {
        throw new Error('This shouldn\'t be called. We override getLastResult to get up-to-date info');
    }
    getLastResult() {
        const resp = new HealthCheckResult(HealthCheckStatus.OK, 'OK', new Date().getTime(), {});
        this.healthChecks.forEach((healthCheck) => {
            const lastResult = healthCheck.getLastResult();
            if (getStatusIndex(lastResult.status) > getStatusIndex(resp.status)) {
                resp.status = lastResult.status;
                resp.message = `Check ${healthCheck.getCheckName()} returned ${resp.status}`;
            }
            resp.data(lastResult);
        });
        return resp;
    }
}
exports.HealthCheckGroup = HealthCheckGroup;
//# sourceMappingURL=HealthCheck.js.map